 /* -*- Mode: c -*-

     Lexer for a minimal programming language "nanoLang"

     Copyright 2015 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

%option noyywrap
%option yylineno

%{
   #include <string.h>
   #include "nanoparse.tab.h"

   int yycol = 1;

#define YY_USER_ACTION {\
      yylloc.first_line = yylloc.last_line =yylineno;   \
      yylloc.first_column = yycol; \
      yycol += yyleng;\
      yylloc.last_column = yycol-1; \
      yycol += yyleng;\
   }
%}

DIGIT      [0-9]
ALPHA      [a-zA-Z]
ALNUM      {ALPHA}|{DIGIT}

INT        {DIGIT}+
ID         {ALPHA}{ALNUM}*


%%

"("  {
      return OPENPAR;}
")"  {
      return CLOSEPAR;}
"*"  {
      return MULT;}
"/"  {
      return DIV;}
"+"  {
      return PLUS;}
"-"  {
      return MINUS;}
"="  {
      return EQ;}
"!=" {
      return NEQ;}
">"  {
      return GT;}
"<"  {
      return LT;}
">=" {
      return GEQ;}
"<=" {
      return LEQ;}
"{"  {
      return OPENCURLY;}
"}"  {
      return CLOSECURLY;}
";"  {
      return SEMICOLON;}
","  {
      return COMA;}


Integer {
   return INTEGER;}
String  {
   return STRING;}
if      {
   return IF;}
else    {
   return ELSE;}

while   {
   return WHILE;}
return  {
   return RETURN;}
print   {
   return PRINT;}

{ALPHA}{ALNUM}* {return IDENT;}

\"[^"]*\"       {return STRINGLIT;}

{DIGIT}+        {return INTLIT;}


#[^\n]*\n { /* Eat comments */}

[\n]    {yycol = 1;}

[\r\t ] { /* Skip  whitespace*/ }


 /* Everything else is an invalid character. */
.   {return ERROR;}

%%
